class IORVechile(object):
    def __init__(self, configuration=None):
        self.configuration = configuration
        self.initialise()
    def initialise(self):
        pass
    def move(self, speed):
        pass
    def steer(self, value):
        pass